@@ -23,7 +23,7 @@ It has no public interface yet.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008 Robin Redeker, all rights reserved.
+Copyright 2008-2009 Robin Redeker, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -62,23 +62,11 @@ sub error {
    $self->response ($code, $msg, $hdr, $content);
 }
 
-sub response {
-   my ($self, $code, $msg, $hdr, $content) = @_;
-   my $res = "HTTP/1.0 $code $msg\015\012";
-   $hdr->{'Expires'}        = $hdr->{'Date'}
-                            = time2str time;
-   $hdr->{'Cache-Control'}  = "max-age=0";
-   $hdr->{'Content-Length'} = length $content;
-   $hdr->{'Connection'}     = $self->{keep_alive} ? 'Keep-Alive' : 'close';
+sub response_done {
+   my ($self) = @_;
 
-   while (my ($h, $v) = each %$hdr) {
-      $res .= "$h: $v\015\012";
-   }
-
-   $res .= "\015\012";
-   $res .= $content;
-
-   $self->{hdl}->push_write ($res);
+   $self->{hdl}->on_drain; # clear any drain handlers
+   (delete $self->{transfer_cb})->() if $self->{transfer_cb};
 
    if ($self->{keep_alive}) {
       $self->push_header_line;
@@ -88,6 +76,73 @@ sub response {
    }
 }
 
+sub response {
+   my ($self, $code, $msg, $hdr, $content) = @_;
+   return unless $self->{hdl};
+
+   my $res = "HTTP/1.0 $code $msg\015\012";
+   $hdr->{'Expires'}        = $hdr->{'Date'}
+                            = time2str time;
+   $hdr->{'Cache-Control'}  = "max-age=0";
+   $hdr->{'Connection'}     = $self->{keep_alive} ? 'Keep-Alive' : 'close';
+
+   $hdr->{'Content-Length'} = length $content
+      if not (defined $hdr->{'Content-Length'}) && not ref $content;
+
+   unless (defined $hdr->{'Content-Length'}) {
+      # keep alive with no content length will NOT work.
+      delete $self->{keep_alive};
+   }
+
+   while (my ($h, $v) = each %$hdr) {
+      $res .= "$h: $v\015\012";
+   }
+
+   $res .= "\015\012";
+
+   if (ref ($content) eq 'CODE') {
+      weaken $self;
+
+      my $chunk_cb = sub {
+         my ($chunk) = @_;
+
+         return 0 unless defined ($self) && defined ($self->{hdl});
+
+         delete $self->{transport_polled};
+
+         if (defined ($chunk) && length ($chunk) > 0) {
+            $self->{hdl}->push_write ($chunk);
+
+         } else {
+            $self->response_done;
+         }
+
+         return 1;
+      };
+
+      $self->{transfer_cb} = $content;
+
+      $self->{hdl}->on_drain (sub {
+         return unless $self;
+
+         if (length $res) {
+            my $r = $res;
+            undef $res;
+            $chunk_cb->($r);
+
+         } elsif (not $self->{transport_polled}) {
+            $self->{transport_polled} = 1;
+            $self->{transfer_cb}->($chunk_cb) if $self;
+         }
+      });
+
+   } else {
+      $res .= $content;
+      $self->{hdl}->push_write ($res);
+      $self->response_done;
+   }
+}
+
 sub _unquote {
    my ($str) = @_;
    if ($str =~ /^"(.*?)"$/) {
@@ -262,11 +317,10 @@ sub _parse_headers {
 sub push_header {
    my ($self, $hdl) = @_;
 
-   $self->{hdl}->unshift_read (regex =>
-      qr<\015?\012\015?\012>, undef, qr<^.*[^\015\012]>,
+   $self->{hdl}->unshift_read (line =>
+      qr{(?<![^\012])\015?\012},
       sub {
          my ($hdl, $data) = @_;
-         $data =~ s/\015?\012$//s;
          my $hdr = _parse_headers ($data);
 
          unless (defined $hdr) {
@@ -336,6 +390,8 @@ sub push_header_line {
 sub do_disconnect {
    my ($self, $err) = @_;
 
+   $self->{transfer_cb}->() if $self->{transfer_cb};
+   delete $self->{transfer_cb};
    delete $self->{req_timeout};
    $self->event ('disconnect', $err);
    delete $self->{hdl};
